<?php
/**
 * Created by PhpStorm.
 * User: Mead
 * Date: 2019/10/8
 * Time: 3:41 PM
 */

namespace App\Handlers;

use App\Models\BikeTraitModel;

class BikeStatusInfoSyncHandler
{
    use BikeTraitModel;
    //用户
    const ROLE_USER = 'user';
    //运维人员
    const ROLE_WORKER = 'worker';
    //系统本身
    const ROLE_SERVER = 'server';
    //绑定人员
    const ROLE_BIND = 'bind';

    private static $redis = '';
    //还车时过期时间
    const CLOSE_BIKE_TTL = 10;

    //正常未骑行车的缓存标识
    const REDIS_BIKE_LOCATION_TAG = 'bike_locations';
    const REDIS_RIDE_BIKE_ORDERS_TAG = 'ride_orders:';

    public function __construct($redis)
    {
        self::$redis = $redis;
    }

    /**
     * 车辆处于骑行状态 (用户订单时（data: 订单id[id],车辆编号[bike_no],区域id[area_id],车辆id[bike_id]）)
     * @param $bike_no
     * @param array $data
     * User: Mead
     */
    public function toBikeRideStatus($role, $bike_no, $data = [])
    {
        $data['role'] = $role;
        self::$redis->set(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, serialize($data));

        $merchant_id = $this->byBikeNoGetMerchantId($bike_no);
        $bike_key = self::REDIS_BIKE_LOCATION_TAG . ":{$merchant_id}";
        //删除未骑行状态
        self::$redis->zrem($bike_key, $bike_no);
    }

    /**
     * 车辆处于等待骑行状态
     * @param $bike_no
     * User: Mead
     */
    public function toBikeWaitRideStatus($bike_no, $lng = 0, $lat = 0)
    {
//        self::$redis->del(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no);
        self::$redis->expire(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no, self::CLOSE_BIKE_TTL);
        $merchant_id = $this->byBikeNoGetMerchantId($bike_no);
        $bike_key = self::REDIS_BIKE_LOCATION_TAG . ":{$merchant_id}";
        self::$redis->geoadd($bike_key, $lng, $lat, $bike_no);
    }

    /**
     * 车辆上线
     * @param $bike_no
     * @param int $lat
     * @param int $lng
     * User: Mead
     */
    public function toBikeOnLineStatus($bike_no, $lng = 0, $lat = 0)
    {
        self::$redis->del([self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no]);
        $merchant_id = $this->byBikeNoGetMerchantId($bike_no);
        $bike_key = self::REDIS_BIKE_LOCATION_TAG . ":{$merchant_id}";
        self::$redis->geoadd($bike_key, $lng, $lat, $bike_no);
    }

    /**
     * 车辆离线
     * @param $bike_no
     * User: Mead
     */
    public function toBikeOffLineStatus($bike_no)
    {
        $merchant_id = $this->byBikeNoGetMerchantId($bike_no);
        $bike_key = self::REDIS_BIKE_LOCATION_TAG . ":{$merchant_id}";
        //删除未骑行状态
        self::$redis->zrem($bike_key, $bike_no);
    }

    /**
     * 获取订单信息
     * @param $bike_no
     * @return mixed
     * User: Mead
     */
    public function getRideBikeOrderInfo($bike_no)
    {
        $order = self::$redis->get(self::REDIS_RIDE_BIKE_ORDERS_TAG . $bike_no);
        if (!$order) return false;
        return unserialize($order);
    }
}